package pinball;

import pinball.*;
import pinball.menuevent.*;
import pinball.io.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.io.*;
import java.util.*;
import java.text.*;

/** 
 * The game options used in the pinball game. 
 * It contains numerous parameters for the system.
 */
public class GameOptions {
	/* The threads to be turned on the program. */
	
	/** Status should post analyze thread run. */
	private static boolean post=true;

	/** Status should analyze thread run. */
	private static boolean analyze=true;
	
	/** Status should strategy thread run. */
	private static boolean strategy=true;

	/** Should the system display an image. */
	private static boolean display=true;
	
	/* Flags to see if playing or recording the game. */
	
	/** Play a game back. */
	private static boolean play=false;
	
	/** Record a game. */
	private static boolean record=false;
	
	/** Number of pixels to increment when scanning the image. */
	private static int pixelrate=1;
	
	/** Display every nth frame on the screen. */
	private static int framerate=1;
	
	/** Number of buffers in the system. */
	private static int buffersize=2;
	
	/** Number of trys the system uses to average the flipper positions. */
	private static int findFlippers=3;
	
	/** 
	 * Is the display in verbose mode. This displays what the system sees,
	 * the difference image. 
	 */
	private static boolean verbose=false;
	
	/** How long from flipper up to the flipper is reset. */
	private static int resetTime=100;

	/** 
	 * The hight from the flipper to the top the flipper test area. 
	 * This is used to make the flipper zone; the area the ball passes through it 
	 * twice to say it was hit.
	 */	
	private	static int flipperThreshold=25;

	/** The Y location for the top of the flipper zone. */
	private	static int flipperZone=-1;

	/** The size of the flipper Zone. */
	private	static int flipperZoneSize=10;

	/** The distance that a ball can move and still be considered a ball.
	This helps filter out noise. */
	private static int moveThreshold=30;

	/** The amount a pixel's index needs to change to be consided it changed. */
	private static int pixelThreshold=10;

	/** The radius around the flipper that are to be ignored. */
	private static int flipperSpace=1;
			
	/** Flag to determine if the system is running. */
	private static boolean running=false;
	
	/** 
	 * The debug level for the program. Current levels are 0 1 2.
	 * The higher the number the more is listed. 0 is off.
	 */
	private static int debug=0;

	/** The min size for the ball. */
	private static int ballMin=1;
	
	/** The max size for the ball. */
	private static int ballMax=30;
		
	/** 
	 * Used to see if ball is out of play. Used by the out of play button 
	 * to force the ball out of play if the system missed it going out. 
	 */
	private static boolean ballOutOfPlay=false;
	
	/** Should the flipper's locations be saved to a file. */
	private static boolean writeFlippersFile=false;
	
	/** Should the flippers locations be read from a file. */
	private static boolean readFlippersFile=false;
	
	/** Name of the flippers file. */
	private static String flippersFile=null;
	
	/* Input file, for flippers. */
	
	/** Used for the flippers file. */
 	private static FileInputStream flip=null;

	/** Used for the flippers file. */
        private static DataInputStream flipin=null;
	
	
	/* Output file, for flippers. */
	
	/** Used for the flippers file. */
	private static FileOutputStream flipto=null;

	/** Used for the flippers file. */
        private static DataOutputStream flipout=null;
	
	/** Should the sytem locate the flippers. False if using file. */
	private static boolean locateFlippers=true;
	
	/** Have the flippers been found. */
	private static boolean foundFlippers=false;
	
	/** Command line arguments. */
	private static String[] arguments;
	
	/** Plunger for the system. */
	private static Plunger plunger;
	
	/** Should the ball be launched. */
	private static boolean launchBall=false;

	/** Used when exiting the system. */
	private static PinballExit exitSystem;
	
	/** Number of ball's to play before ending. */
	private static int ballsToPlay=0;
	
	/** Current ball in play. */
	private static int ballInPlay=0;
	
	/** Name of the log file. */
	private static String logFileName;
	
	/** Port used for data IO. */
	private static String dataPort="/dev/cuaa0";
	
	/** Port used for the network. */
	private static int networkPort=8181;
	
	/** Network Host. */
	private static String networkHost="146.245.1.52";


	/** The device used for data IO. */
	private static DataIO dataIO;
	
	/** Refinding the flippers. */
	private static boolean refindFlippers=false;
	
	/** System is paused. */
	private static boolean pause=false;
	
	/** Client mode. */
	private static boolean clientMode=false;
	
/************************************************************************/
		
	/** Set the running status of the system.
	 *
	 *  @param o The running status.
	 */
	public static void running(boolean o) {
		running=o;
	}
	/**
	 *  Find the running status of the system.
	 *
	 *  @return running state.
	 */
	public static boolean running() {
		return running;
	}	
			
	/**
	 * Set the status of the strategy thread. 
	 *
	 * @param o strategy thread status.
	 */
	public static void strategy(boolean o) {
		strategy=o;
	}
	
	/**
	 * Set the status of the analyze thread. 
	 *
	 * @param o analyze thread status.
	 */
	public static void analyze(boolean o) {
		analyze=o;
	}

	/**
	 * Set the status of the post thread. 
	 *
	 * @param o post thread status.
	 */
	public static void post(boolean o) {
		post=o;
		if(post==false)
			display=false;
	}	
	
	
	/**
	 * Get the display status.
	 *
	 * @return The display status.
	 */
	public static boolean display() {
		return display;
	}
	
	
	/**
	 * Get the post thread status.
	 *
	 * @return The post thread status.
	 */
	public static boolean post() {
		return post;
	}



	/**
	 * Get the strategy thread status.
	 *
	 * @return The strategy thread status.
	 */
	public static boolean strategy() {
		return strategy;
	}

	/**
	 * Get the analyze thread status.
	 *
	 * @return The analyze thread status.
	 */
	public static boolean analyze() {
		return analyze; 
	}
	
	/**
	 * The debug level.
	 *
	 * Current levels are 0 1 2.
	 * The lower the number the more is listed.
	 *
	 * @return The debug level.
	 */    		
	public static int debug() {
		return debug;
	}
	
	/**
	 * Is the system to playback the images.
	 *
	 * @return The play condition.
	 */	
	public static boolean play() {
		return play;
	}

	/**
	 * Is the system to record the images.
	 *
	 * @return The record condition.
	 */	
	public static boolean record() {
		return record;
	}
	
	/**
	 * Set the system up to record the images. 
	 */	
	public static void record_it(){
		record=true;
		play=false;
	}
	
	/**
	 * Set the system up to playback the images. 
	 */	
	public static void play_it() {
		play=true;
		record=false;
	}
	
	/**
	 * Set the verbose status.
	 * This displays what the system sees, the difference image.
	 *
	 * @param v The status.
	 */
	public static void verbose(boolean v) {
		verbose=v;
	}
	  
	/**
	 * Find the verbose status.
	 *
	 * @return The verbose status.
	 */	  
	public static boolean verbose() {
		return verbose;
	}
	  
	/**
	 * Set the system into live mode. That doesn't record or playback images.
	 */	
	public static void live() {
		record=false;
		play=false;
	}

	/**
	 * Get the pixel rate for the system.
	 * Number of pixels to increment when scanning the image.
	 *
	 * @return The pixel rate of the system.
	 */	
	public static int pixelRate() {
		return pixelrate;
	}
	
	/** 
	 * Set the pixel rate for the system.
	 *
	 * @param p pixel rate.
	 */
	public static void pixelRate(int p) {
		pixelrate=p;
	}
	
	/**
	 * Setup the frame rate for the system.
	 * Display every nth frame on the screen.
	 *
	 * @param f the frame rate.
	 */
	public static void frameRate(int f) {
		framerate=f;
		if(framerate<=0)
			display=false;
	}

	/**
	 * Get the frame rate for the system.
	 *
	 * @return The frame rate.
	 */	
	public static int frameRate() {
		return framerate;
	}
	

	/**
	 * Setup the number of buffers.
	 * Number of buffers in the system.
	 * 
	 * @param b Buffer size.
	 */
	public static void bufferSize(int b) {
		buffersize=b;
	}


	/**
	 * Get the buffer size for the sytem.
	 *
	 * @return The buffer size.
	 */
	public static int bufferSize() {
		return buffersize;
	}
	

	/**
	 * Get the number of times to check the flippers.
	 *
	 * @return The number of times to check the flippers.
	 */
	public static int findFlippers() {
		return findFlippers;
	}
	
	/**
	 * Set the number of times to check the flippers.
	 *
	 * @param f amount of checking to do.
	 */
	public static void findFlippers(int f) {
		findFlippers=f;
	}
	
	/**
	 * How long to leave a flipper up before reseting it.
	 *
	 * @return The reset time.
	 */
	public static int resetTime() {
		return resetTime;
	}
	
	/**
	 * Set how long to leave a flipper up before reseting it.
	 *
	 * @param The time to leave a flipper up.
	 */
	public static void resetTime(int t) {
		resetTime=t;
	}

	/**
	 * Set the flipper zone.
	 * The hight from the flipper to the top the flipper test area. 
	 * This area is used to see if the ball passes through it twice it was hit.
	 *
	 * @param f Flipper zone location.
	 */
	public static void flipperZone(int f) {
		flipperZone=f;
	}
	 
	/**
	 * Get the flipper zone.
	 * The Y location for the top of the flipper zone.
	 *
	 * @return The flipper zone location.
	 */	
	public static int flipperZone() {
		return flipperZone;
	}
	 
	/**
	 * Get the flipper threshold.
	 * The hight from the flipper to the top the flipper test area. 
	 * This area is used to see if the ball passes through it twice it was hit.
	 *
	 * @return The flipper threshold.
	 */	 
	public static int flipperThreshold() {
		return flipperThreshold;
	}
		

	/**
	 * Get the move threshold.
	 * The distance that a ball can move and still be considered a ball.
	 * This helps filter out noise.
	 *
	 * @return The move threshold.
	 */	   
	public static int moveThreshold() {
		return moveThreshold;
	}


	/**
	 * Set the move threshold.
	 * The distance that a ball can move and still be considered a ball.
	 * This helps filter out noise.
	 *
	 * @param amount The move threshold value.
	 */	   
	public static void moveThreshold(int amount) {
		moveThreshold=amount;
	}


	/**
	 * Get the minimum size of a ball.
	 *
	 * @return The minimum size of a ball.
	 */	 
	public static int ballMin() {
		return ballMin;
	}
	
	/**
	 * Set the minimum size of a ball.
	 *
	 * @param min The minimum size of a ball.
	 */	 
	public static void ballMin(int min) {
		ballMin=min;
	}

	
	/**
	 * Get the maximum size of a ball.
	 *
	 * @return The maximum size of a ball.
	 */		
	public static int ballMax() {
		return ballMax;
	}
	
	/**
	 * Set the maximum size of a ball.
	 *
	 * @param max The maximum size of a ball.
	 */		
	public static void ballMax(int max) {
		ballMax=max;
	}	
 
 	/**
	 * Get the pixel threshold.
	 * The amount a pixel's index needs to change to be consided it changed.
	 *
	 * @return The pixel threshold.
	 */
	public static int pixelThreshold() {
		return pixelThreshold;
	}
	
	/**
	 * Set the pixel threshold.
	 * The amount a pixel's index needs to change to be consided it changed.
	 *
	 * @param amount The pixel threshold.
	 */
	public static void pixelThreshold(int amount) {
		pixelThreshold=amount;
	}

	/**
	 * Get the flipper zone size.
	 * The size of the area the ball passes through it 
	 * twice to say it was hit.
	 *
	 * @return The size of the flipper zone.
	 */	
	public static int flipperZoneSize() {
		return flipperZoneSize;
	}
	
	/**
	 * Get the radius around the flippers to ignore.
	 *
	 * @return The radius around the flippers to ignore.
	 */	
	public static int flipperSpace() {
		return flipperSpace;
	}
	
	/**
	 * Set the radius around the flippers to ignore.
	 *
	 * @param s The radius.
	 */
	public static void flipperSpace(int s) {
		flipperSpace=s;
	}
	
	/** 
	 * Used to force a line statement of where the program is up to.
	 */
	public static void trace(String s) {
		try { throw new Throwable(s);
		}
		catch (Throwable t) {
			t.printStackTrace();
		}
	}
	
	/**
	 * Tell if the ball was forced out of play.
	 *
	 * @param out The out of play forced status.
	 */
	
	public static void ballOutOfPlay(boolean out) {
		ballOutOfPlay=out;
	}

	/**
	 * See if the ball was forced out of play
	 * 
	 * @return ball's out of play status.
	 */
	public static boolean ballOutOfPlay() {
		return ballOutOfPlay;
	}
	
	/**
	 * Set the file to write the flipper's location.
	 *
	 * @param String flipper file
	 */
	public static void writeFlippersFile(String file) {
		writeFlippersFile=true;
		flippersFile=file;
		System.err.println("Flipper File: "+file);
	}
	
	/**
	 * Check whether to write the flipper's location.
	 *
	 * @return b Write state
	 */
	public static boolean writeFlippersFile() {
		return writeFlippersFile;
	}
	
	
	/**
	 * Set whether to read the flipper's location.
	 *
	 * @param file Flippers file
	 */
	public static void readFlippersFile(String file) {		
		flippersFile=file;
		System.err.println("Flipper File: "+file);
		locateFlippers=false;
	}
	
	/** 
	 * Set whether the system should find the location of the flippers.
	 *
	 * @param flipper location status.
	 */
	public static void locateFlippers(boolean f) {
		locateFlippers=f;
	}
	
	/**
	 * Should the system find the flippers?
	 *
	 * @return Whether the flippers should be found.
	 */
	public static boolean locateFlippers() {
		return locateFlippers;
	}
	
	/**
	 * Get the flipper file for reading.
	 *	
	 * @return flipper file
	 */
	public static DataInputStream getFlippersFileRead() {
		/* Open the flippers file. */
		try {flip = new FileInputStream(flippersFile);}
		catch (Exception e){System.err.println(e);}
	
		try { flipin = new DataInputStream(flip);}
		catch (Exception e){System.err.println(e);}
		return flipin;
	}
	 
	/**
	 * Get the flipper file for writting.
	 *	
	 * @return flipper file
	 */
	public static DataOutputStream getFlippersFileWrite() {
		/* Open the flippers file. */
		try {flipto = new FileOutputStream(flippersFile);}
		catch (Exception e){trace("at1");System.err.println(e);}
	
		try { flipout = new DataOutputStream(flipto);}
		catch (Exception e){trace("at2");System.err.println(e);}
		return flipout;
	}
	
	/**
	 * Set the flipper's found status.
	 *
	 * @param f found flipper's status.
	 */	 
	public static void foundFlippers(boolean f) {
		foundFlippers=f;
	}
	
	/**
	 * Get the flipper's found status.
	 *
	 * @return the flipper's found status.
	 */	
	public static boolean foundFlippers() {
		return foundFlippers;
	}

	/**
	 * Set the command line arguments.
	 *
	 * @param args the command line arguments.
	 */
	public static void setArgs(String[] args) {
		arguments=args;
	}
	
	/**
	 * Get the command line arguments.
	 *
	 * @return the command line arguments.
	 */
	public static String[] getArgs() {
		return arguments;
	}
	
	/**
	 * Set the plunger in the system.
	 *
	 * @param p the plunger in the system.
	 */
	public static void setPlunger(Plunger p) {
		plunger=p;
	}
	
	/**
	 * Get the plunger in the system.
	 *
	 * @return the plunger in the system.
	 */
	public static Plunger getPlunger() {
		return plunger;
	}

	/**
	 * Set the status of whether the system should launch the ball.
	 *
	 * @param l launch status.
	 */
	public static void launchBall(boolean l) {
		launchBall=l;
	}
	
	/**
	 * Get the status of whether the system should launch the ball.
	 *
	 * @return launch status.
	 */	
	public static boolean launchBall() {
		return launchBall;
	}
	
	/**
	 * Set the object used to call when exiting the system.
	 * 
	 * @param exit called when exiting the system.
	 */
	public static void setExit(PinballExit exit) {
		exitSystem=exit;
	}
	
	/** 
      	 * Exit the system.
	 */
	public static void exitSystem() {
		exitSystem.exitSystem();
	}
	
	/**
	 * Set the number of ball's to play before exiting. 
	 *
	 * @param b balls to play.
	 */
	public static void ballsToPlay(int b){
		/* If balls are less than 0, 
		   then the machine will not quit 
		   after a set number of balls.
		*/
		if(b>0)
			ballsToPlay=b;
	}
	
	/** 
	 * Get the number of balls to play.
	 *
	 * @return balls to play.
	 */
	public static int ballsToPlay() {
		return ballsToPlay;
	}
	
	/**
	 * Set ball in play number.
	 *
	 * @param b number of ball in play.
	 */
	public static void ballInPlay(int b) {
		ballInPlay=b;
	}
	
	/**
	 * Get the number of the ball in play.
	 *
	 * @return number of ball in play.
	 */
	public static int ballInPlay() {
		return ballInPlay;
	}
	
	/**
	 * Set the name of the log file for playback. 
	 *
	 * @param
	 */	
	public static void logFileName(String log) {
		logFileName=log;
		play_it();
	}
	
	/**
	 * Get the name of the log file.
	 *
	 * @return the log file's name.
	 */
	public static String logFileName() {
		return logFileName;
	}
	
	/**
	 * Get the port for data IO.
	 *
	 * @return the data port.
	 */
	public static String dataPort() {
		return dataPort;
	}
	
	/**
	 * Set the device for data IO.
	 *
	 * @param data the data IO device.
	 */
	public static void setDataIO(DataIO data) {
		dataIO=data;
	}
	
	/**
	 * Get the device used for data IO.
	 *
	 * @return the data IO device.
	 */
	public static DataIO getDataIO() {
		return dataIO;
	}
	
	/**
	 * Set the refind flippers status.
	 *
	 * @param f the status of refinding the flippers.
	 */
	public static void refindFlippers (boolean f) {
		refindFlippers=f;
	}
	
	
	/**
	 * Get the status of the refinding of the flippers.
	 *
	 * @return refind flippers status.
	 */
	public static boolean refindFlippers () {
		return refindFlippers;
	}
	
	/**
	 * Set the pause state of the system.
	 *
	 * @param p the pause state.
	 */
	public static void pause (boolean p) {
		pause=p;
	}
	
	/**
	 * Get the pause state of the system
	 *
	 * @return the pause state.
	 */
	public static boolean pause() {
		return pause;
	}
	
	/**
	 * Set the system for client mode.
	 */
	public static void clientMode() {
		clientMode=true;
	}
	
	/**
	 * Is the program running in client mode.
	 */
	public static boolean client() {
		return clientMode;
	}
	
	/**
	 * Get the network host.
	 *
	 * @return network host name.
	 */
	public static String networkHost() {
		return networkHost;
	}
	
	/**
	 * Get the network port.
	 *
	 * @return network port number.
	 */
	public static int networkPort() {
		return networkPort;
	}	
}
