package pinball;

import pinball.*;
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.buffer.*;
import pinball.menuevent.*;

import pinball.player.standard.*;

import java.awt.*;
import java.io.*;

/**
 * Starts up the pinball program with the type of player that logs in.
 */

public class JavaQcam {
	/** The main menu of the program. Used to start,exit the program. */			
	private MainMenu menu=null;
	
	/** Used to check if the files are found. */
	private boolean fileFound=false;

	/** Generate the camera. Used to watch the pinball game. */
	private Camera Qcam;

	/** Game options. */
	private Options options=null;
		
	/* Output file, for game options. */	

	/** Output file, for game options. */
	private FileOutputStream out=null;
	
	/** Output file, for game options. */
        private DataOutputStream logout=null;

	/* Input file, for game options. */
 	
	/** Input file, for game options. */
	private FileInputStream in=null;
	
	/** Input file, for game options. */
        private DataInputStream login=null;
       
        /** Name and location of the options file. */
       	private String optionsFile="/tmp/options.dat";
        
        /** The log file. */
       	private PrintStream statsLog=null;
        
        /** Is the log file to be used. */
        private boolean logfile=false;
        
        /** The player to play pinball. */
        private StdPlayer player=null;
	
	/** Name of the flipper file that contains the locations of the flippers. */
	private String flippersFile=null;
	
	/* Input file, for flippers. */
	
	/** Input file, for flippers. */
 	private FileInputStream flip=null;
	
	/** Input file, for flippers. */
        private DataInputStream flipin=null;
	
	/** How the player is going to play. */
	private String how;

	
	/**
	 * Starts the pinball program.
	 *
	 * @param args The command line arguments given.
	 * @param player The player of the game.
	 * @param how How the player will player. Local, Client, or Server.
	 */
	public JavaQcam(String[] args, StdPlayer player, String how) {
		System.err.println("Starting Pinball System.");
		this.player=player;
		Qcam=player.getCamera();
		GameOptions.setArgs(args);				
						
		begin(args,how);
	}
	
	/** 
	 * Start up the program. Takes command line arguments.
	 * @param args The command line arguments given.
	 * @param how How the player will player. Local, Client, or Server.
	*/
	protected void begin(String[] args, String how) {
					
		this.how=how;
		
	        /* Check command line arguments. */
	        for(int i=0;i<args.length;i++) {
			char []command; // Command line argument.
			char []value;	// Value of arguement.
			
			command=args[i].toCharArray();
			value=new char[command.length-2];
						
			/* Get value of command. */
			for(int j=2;j<command.length;j++)
				value[j-2]=command[j];
			/* Check command. */
			if(command[0]=='-') {
				/* Server. */
				if(command[1]=='S') {
					System.err.println("Server mode.");
					/* Set the options file to the server's file. */
					optionsFile="/tmp/soptions.dat";
				}
				
				/* Log file, no video. */
				if(command[1]=='l') {
					logfile=true;
					PinballLog.setPath(new String(value),false);				
				}
				
				/* Log file, with video. */
				if(command[1]=='L') {
					logfile=true;
					PinballLog.setPath(new String(value),true);				
 				}


				/* Flipper file. */
				if(command[1]=='F') {
					flippersFile=new String(value);
				}
			}	
	
		}
	        
	        	       

		/* Open the game options. */
		try {in = new FileInputStream(optionsFile);}
		catch (Exception e){fileFound=false;}
		
		/* Check if the file existed.*/
		if(in!=null)
			fileFound=true;
			
		/* Generate the default game options. */
	        options=new Options(this);       					
	
		/* If the game file is found, read from it. This sets up the options
		previously saved. */
		if(fileFound) {
			try { login = new DataInputStream(in);}
			catch (Exception e){System.err.println("4: "+e);}
			options.read(login);
			//options.setVisible(true);
			//options.toFront();
		}


		/* Setup flippers file is called for. */
		if(flippersFile!=null) {
			fileFound=false;
	
			/* Open the flippers file. */
			try {flip = new FileInputStream(flippersFile);}
			catch (Exception e){fileFound=false;}
		
			/* Set the name of the flipper file to write to.*/
			GameOptions.writeFlippersFile(flippersFile);
			GameOptions.readFlippersFile(flippersFile);
			
			
			try{flip.close();}
			catch (Exception e){}
			
	
		}
						
		System.err.println("Waiting...");
	}					

	
	/** Used by the Options menu to continue with the program. */	
	protected void next() {

		/* Save the game options. */
		try {out = new FileOutputStream(optionsFile);}
		catch (Exception e){System.err.println("1: "+e);}
		try { logout = new DataOutputStream(out);}
		catch (Exception e){System.err.println("2: "+e);}
		options.save(logout);
	
		/* Take away the game options window. */
		options.toBack();
		options.dispose();
					
		/* If recording the log file, create it. */
		if(logfile) {
			PinballLog.setStatsFile(player.id(),1);
			System.err.println("Logging Game");
		}
	
		Qcam.cameraInfo();
				
		/* Make the display. */
		menu=new MainMenu(Qcam,player,how);
		
		/* Start the program. */
		menu.begin();
		
		System.err.println("Program running.");
	}	

}	
		
