package pinball.buffer;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.io.*;
import java.util.*;

/**
 * Used when the image from the camera is to be resized. Basically a flag is
 * set and the new dimention is requested.
 */
public class ResizeInfo{
	/** New width. */
	private int width;
	
	/** New height. */
	private int height;
	
	/** New xoffset. */
	private int xoffset;
	
	/** New yoffset. */
	private int yoffset;

	
	/** Resize flag. Indicates wheither to resize the camera.*/
	private boolean resize=false;
	
	/**
	 * Set the size of the image.
	 *
	 * @param w the width of image.
	 * @param h the height of the image.
	 * @param xo the xoffset of the image.
	 * @param yo the yoffset of the image.
	 */ 
	public synchronized void setSize(int w, int h, int xo, int yo) {
		width=w;
		height=h;
		xoffset=xo;
		yoffset=yo;
		resize=true;
	}
	
	/**
	 * Get the width of the image.
	 *
	 * @returns width of the image.
	 */
	public synchronized int width() {
		return width;
	}

	/**
	 * Get the height of the image.
	 *
	 * @returns height of the image.
	 */
	public synchronized int height() {
		return height;
	}
	
	/**
	 * Get the x offset of the image.
	 *
	 * @returns y offset.
	 */
	public synchronized int xoffset() {
		return xoffset;
	}
	
	/**
	 * Get the y offset of the image.
	 *
	 * @returns y offset.
	 */
	public synchronized int yoffset() {
		return yoffset;
	}
	
	/** 
	 * Check if the camera should be resized. 
	 *
	 * @returns resize flag's value.
	 */
	public boolean resize() {
		return resize;
	}
	
	/**
	 * Set that the camera was resized. 
	 */
	public void resized() {
		resize=false;
	}
}
