package pinball.log;

import java.io.*;
import java.util.*;
import java.text.*;

/** 
 * This reads frame rates in the log file and generates an html file with the statistics. 
 */
public class TimeStampReport {
	/** File that will get the data. */
	private PrintWriter output;

	/** Frame rate. */
	private float avgFrameRate;	
	
	/**
	 * Creates the TimeStamp reports for the log files.
	 *
	 * @param out the output file.
	 */
	public TimeStampReport(PrintWriter out){
		output=out;
	}
	
	/**
	 * Finds the frame rate statistics.
	 *
	 * @param tokens the line of tokens to process.
	 */
	public void process(StringTokenizer tokens) {
		String command;

		while(tokens.hasMoreTokens()) {
			command=tokens.nextToken(" ");
			check(command);
		}
	}
	
	
	/**
	 * Analyzes the frame rate information.
	 *
	 * @param data the data from the log file.	   
	 */
	private void check(String data) {
		StringTokenizer tokens;
		String command;
			
		
		tokens = new StringTokenizer(data,"=");
		
		command=tokens.nextToken();
		
		if(command.equals("avgFrameRate")) {
			avgFrameRate=new Float(tokens.nextToken()).floatValue();
		}
	}
	
	
	/**
	 * Writes the information about the frame rate.
	 */
 	public void writeInfo() {		
		float frameAdj=10;

		/* Table of average frame rate. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		output.println("\t<TR>");
		output.println("\t\t<TD>Average Frame Rate (fps)</TD>");
		output.println("\t</TR>");
		
		output.println("\t<TR>");
		output.println("\t\t<TD>"+avgFrameRate+"</TD>");
		output.println("\t</TR>");
		output.println("</TABLE><BR>");
	}
}
