package pinball.player.adjustable;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/** Draws the image and image information to the screen. */
public class AdjustableDrawingCanvas extends DrawingCanvas{

	
	/** When the ball is in this polygon, the left flipper flips. */
	protected Polygon leftAdjust;
	
	/** When the ball is in this polygon, the right flipper flips. */	
	protected Polygon rightAdjust;
	
	/** Used to make the adjustments for the polygons for the flippers. */
	protected AdjustableFlippers adjustments;	

	/** The special bufferinfo. */
	protected AdjustableBufferInfo adjustableBuffinfo;


	/** 
	 * Initalize the canvas. 
	 *
	 * @param w width of the image.
	 * @param h height of the image.
	 */
	public AdjustableDrawingCanvas(int w, int h){
		super(w,h);
	}


	/** 
	 * Setup the adjustable flippers.
	 * 
	 * @param adjustments the flippers to setup. 
	 */
	public void setAdjustments(AdjustableFlippers adjustments) {
		addMouseListener(adjustments);
		addKeyListener(adjustments);
		
		this.adjustments=adjustments;
		adjustments.setParent(this);
		System.err.println("Listening");
	}


	/**
	 * Update the image's data. Draw to the off screen image.
	 */
	public void updateFrame (BufferInfo buffinfo) {
		adjustableBuffinfo = (AdjustableBufferInfo)buffinfo;
		/* Get the flipper adjustments. */	
		leftAdjust=adjustableBuffinfo.getAdjustableFlippers().getLeft();
		rightAdjust=adjustableBuffinfo.getAdjustableFlippers().getRight();

		/* The location of the ball. */
		int x=buffinfo.blip().x();
		int y=buffinfo.blip().y();
		
		/* Make the image.*/
	        MemoryImageSource mis = 
        		new MemoryImageSource(buffinfo.width(),buffinfo.height(),
  		   	   		model, buffinfo.get(), 0, buffinfo.width());

	        /* The image from the camera. */
		Image myImage = createImage(mis);

		if(GameOptions.debug()>=2)
	     		  System.err.println("Frame Updated.");
		
		/* Draw new image in the frameBuffer. */

		/* Clear the background if the buffer calls for it. 
		   Used when resizing to erase the background.*/
		if(buffinfo.clearing()) {
			graphicsContext.clearRect(0,0,buffinfo.width(),buffinfo.height());
			buffinfo.clearing(false);
		}

		/* Draw the image of the game. */
		graphicsContext.drawImage(myImage,buffinfo.XOffset(),buffinfo.YOffset(),buffinfo.width(),buffinfo.height(),this);

		
		/* Draw the cross for the ball's location. */
		if(x>-1 && y>-1) {
			x+=buffinfo.XOffset();
			y+=buffinfo.YOffset();

			if(GameOptions.verbose())
				graphicsContext.setColor(Color.blue);
			else graphicsContext.setColor(Color.yellow);
			graphicsContext.drawLine(x,y-5,x,y+5);
			graphicsContext.drawLine(x+5,y,x-5,y);
		}
			
		/* Draw the flipper zone. */	
		if(GameOptions.flipperZone()!=-1) {	
			graphicsContext.setColor(Color.yellow);
			graphicsContext.drawRect(buffinfo.XOffset(),GameOptions.flipperZone(),buffinfo.width()-1,GameOptions.flipperZoneSize());
		}
 		
		/* The polygon for the left flipper's zone. */
		if(leftAdjust!=null) {
			graphicsContext.setColor(Color.cyan);
			graphicsContext.drawPolygon(leftAdjust);

		}
		
		/* The polygon for the left flipper's zone. */
		if(rightAdjust!=null) {
			graphicsContext.setColor(Color.green);
			graphicsContext.drawPolygon(rightAdjust);
		}
	
		/* Have the strategy draw it's lines so you can see the decision process. */
		if(stub!=null) 
			stub.strategyDraw(graphicsContext,buffinfo.width(),buffinfo.height());		

		paint(getGraphics());
	}


}
