package pinball.player.network;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.net.*;
/**
 * Used to listen for commands from the client side. The remote stub sends these
 * commands.
 */
public class NetworkListener extends Thread {
	/** The server that receives the data. */
	protected MyServer server;
	
	/** The stub to control. */
	protected Stub stub;
		
	/** The strategy who plays the game. */
	protected Strategy strategy;
		
	/**
	 * Setup the network listener. Get data from the client.
	 *
	 * @param server the server who gets the info.
	 * @param stub the stub to control the machine.
	 */
	public NetworkListener (MyServer server, Stub stub){
		this.server=server;
		this.stub=stub;
	}

	/**
	 * Name of the object.
	 */
	public String toString() {
		return "NetworkListener";
	}


	/**
	 * Listen for commands and follow them out.
	 */
	public void run() {
		while(true) {
			listening();
			System.err.println("NetworkListener Offline");
			server.offline();										
			server.connection();
		}
	}

	/**
	 * Keep listening for commands and follow them out.
	 */	
	public void listening() {
		System.err.println("NetworkListener is online...");
		String data;
		StringTokenizer tokens;
		String command;
		
		while((data=server.receive())!=null) {
			command=null; 			
			tokens = new StringTokenizer(data);
		
			if(tokens.countTokens()>1)
				command=tokens.nextToken();
			else command=data;
				
//			System.err.println("Network listing");
			if(command!=null) {
//				System.err.println(data);
				if(command.equals("leftFlip"))
					stub.leftFlipperState(true);
				else if(command.equals("rightFlip"))
					stub.rightFlipperState(true);
				else if(command.equals("leftReset"))
					stub.leftFlipperState(false);
				else if(command.equals("rightReset"))
					stub.rightFlipperState(false);
				else if(command.equals("releaseLock"))
					stub.releaseLock(new Integer(tokens.nextToken()).intValue());
				else if(command.equals("RESIZE")) {  
					stub.resize(new Integer(tokens.nextToken()).intValue(),
						new Integer(tokens.nextToken()).intValue(),
						new Integer(tokens.nextToken()).intValue(),
						new Integer(tokens.nextToken()).intValue());
				}

				else if(command.equals("flipperSpace"))
					GameOptions.flipperSpace(new Integer(tokens.nextToken()).intValue());
				
				else if(command.equals("topBottom")) {
					stub.topBottom(new Integer(tokens.nextToken()).intValue(),new Integer(tokens.nextToken()).intValue());
//					System.err.println("TOP "+stub.top()+" BOTTOM "+stub.bottom());
					
				}
				
				else if(command.equals("frame"))
					stub.sendFrame(true);
				
				else if(command.equals("LOG"))
					getLogInfo(tokens);
					
				else if(command.equals("HEADER"))
					getHeaderInfo(tokens);

				else if(command.equals("PING"))
					ping();
					
				else if(command.equals("QUIT")) {
					server.offline();
					stub.releaseAllBuffers();					
					break;
				}					
				
				else if(command.equals("releaseAllBuffers")) {
					stub.releaseAllBuffers();
				}		
				
				/* Check with extended players for any new commands. */	
				else if(processCommand(command))
					;
					
				/* Invalid command. */																	
				else System.err.println("BAD COMMAND: "+command);	
			}
			
			/* Let another thread go. */
			Thread.yield();			
		}
		
		System.err.println("Network has stopped listing");
	}		
	
	
	/**
	 * Process the command from the network. Must return true or false to
	 * indicate if the command was processed.
	 *
	 * @param command the command from the network.
	 * @return the indication of weither the command was processed.
	 */
	 public boolean processCommand(String command) {
	 	return false;	 
	 }
		
	/**
	 * Respond to ping with pong.
	 */
	public void ping() {		
		server.transmit("PONG");
	}
	
	/**
	 * Get the log info.
	 *
	 * @param tokens the data from the command.
	 */
	public void getLogInfo(StringTokenizer tokens) {
		/* Number of tokens in the data. */
		int amount=tokens.countTokens();
		
		/* Inital value is the frame number. */
		int frame=new Integer(tokens.nextToken()).intValue();
	
		String info="";
			

		/* Get the remaining words, except the priority number. */
		for(int i=0;i<(amount-2);i++) 
			info+=" "+tokens.nextToken();
		
		int priority=new Integer(tokens.nextToken()).intValue();
		
		stub.logit(frame,info,priority,this);
	}
	
	/**
	 * Get the header info.
	 *
	 * @param tokens the data from the command.
	 */
	public void getHeaderInfo(StringTokenizer tokens) {
		int amount=tokens.countTokens();
		
		/* Inital value is the frame number. */
		String info=tokens.nextToken();

		/* Get the remaining words, except the priority number. */
		for(int i=0;i<(amount-1);i++) 
			info+=" "+tokens.nextToken();
		
		stub.headerInfo(info);
	}
		
		
}	
